@@ -1,14 +1,21 @@
 import math
+from typing import Optional
 
 import torch
 import torch.nn.functional as F
 
-__all__ = ["sigmoid_focal_loss", "soft_jaccard_score", "soft_dice_score", "wing_loss"]
+__all__ = ["focal_loss_with_logits", "sigmoid_focal_loss", "soft_jaccard_score", "soft_dice_score", "wing_loss"]
 
 
-def sigmoid_focal_loss(
-    input: torch.Tensor, target: torch.Tensor, gamma=2.0, alpha=0.25, reduction="mean"
-):
+def focal_loss_with_logits(
+    input: torch.Tensor,
+    target: torch.Tensor,
+    gamma=2.0,
+    alpha: Optional[float] = 0.25,
+    reduction="mean",
+    normalized=False,
+    reduced_threshold: Optional[float] = None,
+) -> torch.Tensor:
     """Compute binary focal loss between target and output logits.
 
     See :class:`~pytorch_toolbelt.losses.FocalLoss` for details.
@@ -23,21 +30,32 @@ def sigmoid_focal_loss(
             and :attr:`reduce` are in the process of being deprecated, and in the meantime,
             specifying either of those two args will override :attr:`reduction`.
             'batchwise_mean' computes mean loss per sample in batch. Default: 'mean'
-
+        normalized (bool): Compute normalized focal loss (https://arxiv.org/pdf/1909.07829.pdf).
+        reduced_threshold (float, optional): Compute reduced focal loss (https://arxiv.org/abs/1903.01347).
     References::
 
         https://github.com/open-mmlab/mmdetection/blob/master/mmdet/core/loss/losses.py
     """
     target = target.type(input.type())
 
-    logpt = -F.binary_cross_entropy_with_logits(input, target, reduction="none")
-    pt = torch.exp(logpt)
+    logpt = F.binary_cross_entropy_with_logits(input, target, reduction="none")
+    pt = torch.exp(-logpt)
 
     # compute the loss
-    loss = -((1 - pt).pow(gamma)) * logpt
+    if reduced_threshold is None:
+        focal_term = (1 - pt).pow(gamma)
+    else:
+        focal_term = ((1.0 - pt) / reduced_threshold).pow(gamma)
+        focal_term[pt < reduced_threshold] = 1
+
+    loss = focal_term * logpt
 
     if alpha is not None:
-        loss = loss * (alpha * target + (1 - alpha) * (1 - target))
+        loss *= alpha * target + (1 - alpha) * (1 - target)
+
+    if normalized:
+        norm_factor = focal_term.sum() + 1e-5
+        loss /= norm_factor
 
     if reduction == "mean":
         loss = loss.mean()
@@ -49,56 +67,18 @@ def sigmoid_focal_loss(
     return loss
 
 
-def reduced_focal_loss(
-    input: torch.Tensor,
-    target: torch.Tensor,
-    threshold=0.5,
-    gamma=2.0,
-    reduction="mean",
-):
-    """Compute reduced focal loss between target and output logits.
-
-    See :class:`~pytorch_toolbelt.losses.FocalLoss` for details.
-
-    Args:
-        input: Tensor of arbitrary shape
-        target: Tensor of the same shape as input
-        reduction (string, optional): Specifies the reduction to apply to the output:
-            'none' | 'mean' | 'sum' | 'batchwise_mean'. 'none': no reduction will be applied,
-            'mean': the sum of the output will be divided by the number of
-            elements in the output, 'sum': the output will be summed. Note: :attr:`size_average`
-            and :attr:`reduce` are in the process of being deprecated, and in the meantime,
-            specifying either of those two args will override :attr:`reduction`.
-            'batchwise_mean' computes mean loss per sample in batch. Default: 'mean'
-
-    References::
-
-        https://arxiv.org/abs/1903.01347
-    """
-    target = target.type(input.type())
-
-    logpt = -F.binary_cross_entropy_with_logits(input, target, reduction="none")
-    pt = torch.exp(logpt)
-
-    # compute the loss
-    focal_reduction = ((1.0 - pt) / threshold).pow(gamma)
-    focal_reduction[pt < threshold] = 1
-
-    loss = -focal_reduction * logpt
-
-    if reduction == "mean":
-        loss = loss.mean()
-    if reduction == "sum":
-        loss = loss.sum()
-    if reduction == "batchwise_mean":
-        loss = loss.sum(0)
-
-    return loss
+# TODO: Mark as deprecated and emit warning
+sigmoid_focal_loss = focal_loss_with_logits
 
 
-def soft_jaccard_score(
-    y_pred: torch.Tensor, y_true: torch.Tensor, smooth=0.0, eps=1e-7, dims=None
-) -> torch.Tensor:
+# TODO: Mark as deprecated and emit warning
+def reduced_focal_loss(input: torch.Tensor, target: torch.Tensor, threshold=0.5, gamma=2.0, reduction="mean"):
+    return focal_loss_with_logits(
+        input, target, alpha=None, gamma=gamma, reduction=reduction, reduced_threshold=threshold
+    )
+
+
+def soft_jaccard_score(y_pred: torch.Tensor, y_true: torch.Tensor, smooth=0.0, eps=1e-7, dims=None) -> torch.Tensor:
     """
 
     :param y_pred:
@@ -124,13 +104,11 @@ def soft_jaccard_score(
         cardinality = torch.sum(y_pred + y_true)
 
     union = cardinality - intersection
-    jaccard_score = (intersection + smooth) / (union + smooth + eps)
+    jaccard_score = (intersection + smooth) / (union.clamp_min(eps) + smooth)
     return jaccard_score
 
 
-def soft_dice_score(
-    y_pred: torch.Tensor, y_true: torch.Tensor, smooth=0, eps=1e-7, dims=None
-) -> torch.Tensor:
+def soft_dice_score(y_pred: torch.Tensor, y_true: torch.Tensor, smooth=0, eps=1e-7, dims=None) -> torch.Tensor:
     """
 
     :param y_pred:
@@ -153,17 +131,11 @@ def soft_dice_score(
     else:
         intersection = torch.sum(y_pred * y_true)
         cardinality = torch.sum(y_pred + y_true)
-    dice_score = (2.0 * intersection + smooth) / (cardinality + smooth + eps)
+    dice_score = (2.0 * intersection + smooth) / (cardinality.clamp_min(eps) + smooth)
     return dice_score
 
 
-def wing_loss(
-    prediction: torch.Tensor,
-    target: torch.Tensor,
-    width=5,
-    curvature=0.5,
-    reduction="mean",
-):
+def wing_loss(prediction: torch.Tensor, target: torch.Tensor, width=5, curvature=0.5, reduction="mean"):
     """
     https://arxiv.org/pdf/1711.06753.pdf
     :param prediction:
