@@ -8,7 +8,7 @@ from torch.nn.modules.loss import _Loss
 
 from .functional import soft_jaccard_score
 
-__all__ = ["JaccardLoss"]
+__all__ = ["JaccardLoss", "BINARY_MODE", "MULTICLASS_MODE", "MULTILABEL_MODE"]
 
 BINARY_MODE = "binary"
 MULTICLASS_MODE = "multiclass"
@@ -21,19 +21,12 @@ class JaccardLoss(_Loss):
     It supports binary, multi-class and multi-label cases.
     """
 
-    def __init__(
-        self,
-        mode: str,
-        classes: List[int] = None,
-        log_loss=False,
-        from_logits=True,
-        smooth=0,
-        eps=1e-7,
-    ):
+    def __init__(self, mode: str, classes: List[int] = None, log_loss=False, from_logits=True, smooth=0, eps=1e-7):
         """
 
         :param mode: Metric mode {'binary', 'multiclass', 'multilabel'}
-        :param classes: Optional list of classes that contribute in loss computation; By default, all channels are included.
+        :param classes: Optional list of classes that contribute in loss computation;
+        By default, all channels are included.
         :param log_loss: If True, loss computed as `-log(jaccard)`; otherwise `1 - jaccard`
         :param from_logits: If True assumes input is raw logits
         :param smooth:
@@ -43,9 +36,7 @@ class JaccardLoss(_Loss):
         super(JaccardLoss, self).__init__()
         self.mode = mode
         if classes is not None:
-            assert (
-                mode != BINARY_MODE
-            ), "Masking classes is not supported with mode=binary"
+            assert mode != BINARY_MODE, "Masking classes is not supported with mode=binary"
             classes = to_tensor(classes, dtype=torch.long)
 
         self.classes = classes
@@ -65,10 +56,12 @@ class JaccardLoss(_Loss):
 
         if self.from_logits:
             # Apply activations to get [0..1] class probabilities
+            # Using Log-Exp as this gives more numerically stable result and does not cause vanishing gradient on
+            # extreme values 0 and 1
             if self.mode == MULTICLASS_MODE:
-                y_pred = y_pred.softmax(dim=1)
+                y_pred = y_pred.log_softmax(dim=1).exp()
             else:
-                y_pred = y_pred.sigmoid()
+                y_pred = F.logsigmoid(y_pred).exp()
 
         bs = y_true.size(0)
         num_classes = y_pred.size(1)
@@ -89,12 +82,10 @@ class JaccardLoss(_Loss):
             y_true = y_true.view(bs, num_classes, -1)
             y_pred = y_pred.view(bs, num_classes, -1)
 
-        scores = soft_jaccard_score(
-            y_pred, y_true.type(y_pred.dtype), self.smooth, self.eps, dims=dims
-        )
+        scores = soft_jaccard_score(y_pred, y_true.type(y_pred.dtype), self.smooth, self.eps, dims=dims)
 
         if self.log_loss:
-            loss = -torch.log(scores)
+            loss = -torch.log(scores.clamp_min(self.eps))
         else:
             loss = 1 - scores
 
@@ -103,8 +94,8 @@ class JaccardLoss(_Loss):
         # NOTE: A better workaround would be to use loss term `mean(y_pred)`
         # for this case, however it will be a modified jaccard loss
 
-        mask = (y_true.sum(dims) > 0).float()
-        loss = loss * mask
+        mask = y_true.sum(dims) > 0
+        loss *= mask.float()
 
         if self.classes is not None:
             loss = loss[self.classes]
