@@ -1,5 +1,7 @@
 from __future__ import absolute_import
 
-from .metrics import *
-from .visualization import *
 from .criterions import *
+from .metrics import *
+from .opl import *
+from .visualization import *
+from .utils import *
