@@ -51,14 +51,11 @@ def auto_file(filename: str, where: str = ".") -> str:
 
     files = list(glob.iglob(os.path.join(where, "**", filename), recursive=True))
     if len(files) == 0:
-        raise FileNotFoundError(
-            "Given file could not be found with recursive search:" + filename
-        )
+        raise FileNotFoundError("Given file could not be found with recursive search:" + filename)
 
     if len(files) > 1:
         raise FileNotFoundError(
-            "More than one file matches given filename. Please specify it explicitly:\n"
-            + "\n".join(files)
+            "More than one file matches given filename. Please specify it explicitly:\n" + "\n".join(files)
         )
 
     return files[0]
